#!/bin/csh
##############################################################################
#	Program: Illumina sequencing reads pre-processing unit
#	Input: Sequencing reads in fq1 format
#	Input parameters: File in Fq1 format, Size of chunk, S3 bucket, S3 directory	
#	Written: Chandra Sekhar Pedamallu
#	Copyright@DFCI, The Broad Institute
#############################################################################

@ n_para=$#

# Reading environmental variables for running job from cluster.config and job.config
set para = `awk -f readconfig.awk T0=cluster.config T01=job.config < .empty.lst`
set fq1 = $1

echo $fq1 > .tmp
set namefile = `awk '{ns=split($1, x, "/"); print x[ns];}' .tmp`

rm .tmp

set Java = $para[1]
set s3 = $para[3] # S3 Command line runs
set scluster1 = $para[9] #Size of cluster
set rbucket = $para[13] #Reads bucket

@ scluster2=$scluster1
@ tmp1=1
@ scluster = $scluster2 + $tmp1
#echo $scluster

set ext = ".fq1"
set sone = "ONE"

#set nreads = $2

if($n_para < 1) then
    echo "Usage: Preprocessed_Reads.com <Sequence reads in Fq1> <Reads S3 Bucket i.e. ami-test>"
    echo "Please check the number of argument"
    exit
endif

# S3 Bucket availability
set pre = `$s3/s3cmd ls s3://$rbucket | wc -l`

if($pre[1] > 0) then
	echo "S3 Bucket is already available...."$rbucket
	echo "Please remove it using S3cmd or change the Bucket name"
	exit
else
	$s3/s3cmd mb s3://$rbucket
endif

$Java/javac *.java

#Identifying the Chunk size in FQ1 read file
set n_reads = `wc -l < $fq1`
@ nreads1 = $n_reads

@ chunksize = $nreads1 / $scluster

if($chunksize > 1250000) then
	@ ntsk=2
	@ rounded=1
	@ ntasks=$ntsk * $scluster
	@ chunksize = $rounded + ($nreads1 / $ntasks)

endif
	
if($chunksize > 1250000) then
	@ ntsk=3
	@ rounded=1
	@ ntasks=$ntsk * $scluster
	@ chunksize = $rounded + ($nreads1 / $ntasks)
endif


#echo $chunksize $fq1
#echo $ext

# Splitting reads of specified chunk sizes
echo "Splitting reads of specified "$chunksize
$Java/java FileSplitter $fq1 $chunksize $ext $sone $namefile

set folder = $1"_spt"

set nfile1 = `ls -l $folder/*.fq1 | wc -l`
@ nfile = $nfile1
@ i = 1
#echo $nfile
#echo "Hello"

while($i <= $nfile)
	echo "Conversion of file : "$i" into cloud compatible format"

	$Java/java FQone2Fastq $folder/$namefile"_"$i"_.fq1" $folder/$namefile"_"$i".fastq"
	gzip $folder/$namefile"_"$i".fastq"
	
	echo s3://$rbucket/$namefile"_"$i".fastq.gz" > $folder/"input"$i".local"

	@ i++
end

/bin/rm $folder/*.fq1

# Upload into S3 Bucket
echo "Uploading into S3 Bucket "$rbucket
$s3/s3cmd put $folder/*.* s3://$rbucket

echo "For confirmation of successul upload, Please check  "$rbucket

/bin/rm -r $folder
