
import java.io.*;
import java.util.ArrayList;

/**
 * Parses a text file into an ArrayList of Strings.

 * @author kostic@broadinstitute.org
 * @since 2010-08-12
 */
public class BasicParser {

	private String inputFile;
	private ArrayList<String> contents= new ArrayList<String>();
	
	public BasicParser(String in) throws IOException {
		inputFile= in;
		this.parseFile();
		
	}
	
	/**
	 * Run this method on the BasicParser object. This is required for all applications, and
	 * has been added to the constructor.
	 * @return none
	 */
	public void parseFile() throws IOException {
		FileReader fr1=new FileReader(inputFile);
		BufferedReader br=new BufferedReader(fr1);
		
		String strLine;
		while ((strLine = br.readLine()) != null)   {
			contents.add(strLine);
		}
		fr1.close();
	}
	
	/**
	 * Run this method on the BasicParser object. The method returns the line in the file containing
	 * the indicated sub-String.
	 * @return String: line containing match
	 */
	public String stringMatcher(String toFind) {
		String lineFound = null;
		for (String l : contents) {
			if (l.contains(toFind)) {
				lineFound= l;
			}
		}
		return lineFound;
	}
	
	/**
	 * Run this method on the BasicParser object. If the String is found, the method returns true.
	 * @return boolean: true if line is found
	 */
	public boolean isStringFound(String toFind) {
		boolean found= false;
		String lineFound = "error";
		for (String l : contents) {
			if (l.contains(toFind)) {
				lineFound= l;
			}
		}
		if ( !lineFound.equals("error") ) {
			found= true;
		}
		return found;
	}
	
	/**
	 * Splits the input line into an array of Strings.
	 * @return String[]: split line
	 */
	public static String[] splitLine(String toSplit, String delim) {
		String[] columns;
		columns= toSplit.split(delim);
		return columns;
	}
	
	/**
	 * Splits the input line into an array of Strings then returns the specified column of the array.
	 * @return String: column in array
	 */
	public static String splitLineAndReturnCol(String toSplit, String delim, int colNum) {
		String column;
		String[] columns;
		columns= toSplit.split(delim);
		column= columns[colNum];
		return column;
	}
	
	
	
	
	//Setters and Getters
	
	/**
	 * Return file as an ArrayList of Strings.
	 * @return ArrayList<String>: all contents of file
	 */
	public ArrayList<String> getContents() {
		return contents;
	}
}
