
import java.util.ArrayList;

public class Genus {

	private String genus;
	private ArrayList<SequenceEntry> matchingReads= new ArrayList<SequenceEntry>();
	private String mostCommonAccession; //method must be run to find the most common accession among this list of SequenceEntry objects
	private String corrGenus;			//this genus name corresponds to that accession
	private String corrSpecies;			//this species name corresponds to that accession
	private int corrGenomeSize;			//" "
	private int corrReadLength;			//" "

	public Genus(String genusName, SequenceEntry seq) {
		setGenus(genusName);
		matchingReads.add(seq);
	}

	private void findMostCommonAccession() { 	//this method will find the accession # that is most common amongst this set of SequenceEntry objects
		mostCommonAccession= "errorError";		//this method must be run
		corrGenus= "errorError";
		corrSpecies= "errorError";

		ArrayList<String> accessionNames= new ArrayList<String>();
		ArrayList<MostCommonAccession> mcaArray= new ArrayList<MostCommonAccession>();
		for ( SequenceEntry seq : matchingReads ) {
			ArrayList<String> accessions= seq.getAccessionCodes();
			for ( String accession : accessions) {
				if ( !accessionNames.contains(accession) ) {
					accessionNames.add(accession);
					MostCommonAccession mca= new MostCommonAccession(accession);
					mcaArray.add(mca);
				} else {
					MostCommonAccession mca= mcaArray.get(accessionNames.indexOf(accession));
					mca.incAccessionCount();
				}
			}
		}

		int bestMcaCount= 0;
		for ( MostCommonAccession mca : mcaArray) {
			if ( mca.getAccessionCount() > bestMcaCount) {
				bestMcaCount= mca.getAccessionCount();
				mostCommonAccession= mca.getAccession();
			}
		}

		while ( corrGenus.equals("errorError") && corrSpecies.equals("errorError") ) {
			for ( SequenceEntry seq : matchingReads) {
				ArrayList<String> accessionAL= seq.getAccessionCodes();
				ArrayList<String> genusAL= seq.getGenuses();
				ArrayList<String> speciesAL= seq.getSpecies();
				ArrayList<Integer> genomeSizeAL= seq.getGenomeSizes();
				int readLength= seq.getReadLength();
				for (String accession : accessionAL) {
					if (accession.equals(mostCommonAccession)) {
						corrGenus= genusAL.get(accessionAL.indexOf(accession));
						corrSpecies= speciesAL.get(accessionAL.indexOf(accession));
						corrGenomeSize= genomeSizeAL.get(accessionAL.indexOf(accession));
						corrReadLength= readLength;
					}
				}
			}
		}
	}

	//now, find a way to use the Species class to calucate the genome coverage of the most common species in this genus
	public Species getSpeciesCalculations() {
		this.findMostCommonAccession();
		Species s= new Species(corrGenus, corrSpecies, corrGenomeSize, corrReadLength);
		s.setNumReads(0);
		for ( SequenceEntry se : matchingReads) {
			for ( String ac : se.getAccessionCodes() ) {
				if ( ac.equals(mostCommonAccession)) {
					s.addGenomeLoc(se.getGenomeLocs().get(se.getAccessionCodes().indexOf(ac)));
					s.setNumReads(s.getNumReads() + 1);
					break;
				}
			}
		}
		return s;
	}




	private class MostCommonAccession {
		private String accession;
		private int accessionCount;

		public MostCommonAccession(String a) {
			setAccession(a);
			setAccessionCount(1);
		}

		public void incAccessionCount() {
			this.accessionCount++;
		}

		public void setAccession(String accession) {
			this.accession = accession;
		}

		public String getAccession() {
			return accession;
		}

		public void setAccessionCount(int accessionCount) {
			this.accessionCount = accessionCount;
		}

		public int getAccessionCount() {
			return accessionCount;
		}
	}







	//Getters and Setters

	public void setGenus(String genus) {
		this.genus = genus;
	}

	public String getGenus() {
		return genus;
	}

	public void addToMatchingReads(SequenceEntry matchingRead) {
		this.matchingReads.add(matchingRead);
	}

	public ArrayList<SequenceEntry> getMatchingReads() {
		return matchingReads;
	}
}
