/*
 * Details: Run RAM file.
 * Author: Chandra Sekhar Pedamallu, Alex Kostic
 * Date: 10-2-10
 * 
 */
import java.io.*;
import java.util.ArrayList;

public class RunRAM {

	public static void main(String args[]) throws FileNotFoundException {

		//Timer
		long start = System.currentTimeMillis();

		String inputFile=  	args[0]; //resultsHitTable.txt
		String outputFile= 	args[1];
		String genomeTable=	args[2]; // /xchip/pasteur/AlexK/databases/BacterialGenomes_info/complete_microbial_genomes_list_ncbi_condensed.txt

		double identityRatioCutOff= 0.9;
		double queryCoverageRatioCutOff= 0.9;

		try{
			BufferedWriter out = new BufferedWriter(new FileWriter(outputFile));

			ArrayList<SequenceEntry> seqs= new ArrayList<SequenceEntry>();

			String currentReadName = "arbitrary";
			int currentIndex= -1;

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tParsing through hitTable.txt file...");

			FileReader fr1=new FileReader(inputFile);
			BufferedReader br=new BufferedReader(fr1);
			
			String strLine;
			while ((strLine = br.readLine()) != null)   {
				String queryName= BasicParser.splitLineAndReturnCol(strLine, "\t", 0);
				double eValue= Double.parseDouble(BasicParser.splitLineAndReturnCol(strLine, "\t", 8));
				int readLength= Integer.parseInt(BasicParser.splitLineAndReturnCol(strLine, "\t", 1));
				double identityRATIO= Double.parseDouble(BasicParser.splitLineAndReturnCol(strLine, "\t", 13));
				double queryCoverageRATIO= Double.parseDouble(BasicParser.splitLineAndReturnCol(strLine, "\t", 14));
				int genomeSize= Integer.parseInt(BasicParser.splitLineAndReturnCol(strLine, "\t", 6));
				int hitFrom= Integer.parseInt(BasicParser.splitLineAndReturnCol(strLine, "\t", 9));
				int hitTo= Integer.parseInt(BasicParser.splitLineAndReturnCol(strLine, "\t", 10));
				int genomeLoc;
				if ( hitFrom < hitTo )
					genomeLoc= hitFrom;
				else
					genomeLoc= hitTo;
				String accessionCode= BasicParser.splitLineAndReturnCol(strLine, "\t", 5);
				String refDef=  BasicParser.splitLineAndReturnCol(strLine, "\t", 4);
				String genusName= BasicParser.splitLineAndReturnCol(refDef, " ", 0);
				String speciesName= BasicParser.splitLineAndReturnCol(refDef, " ", 1);				

				if (!currentReadName.equals(queryName)) {
					currentIndex++;
					currentReadName= queryName;
					SequenceEntry seqNew= new SequenceEntry(queryName, eValue, readLength);
					seqs.add(seqNew);
					seqNew.addToeVals(eValue);
					seqNew.addToIdRatios(identityRATIO);
					seqNew.addToQueryCovRatios(queryCoverageRATIO);
					seqNew.addToGenomeLocs(genomeLoc);
					seqNew.addToGenomeSizes(genomeSize);
					seqNew.addToAccessionCodes(accessionCode);
					seqNew.addToGenuses(genusName);
					seqNew.addToSpecies(speciesName);
				} else {
					SequenceEntry currentSeqEntry= seqs.get(currentIndex);
					currentSeqEntry.incrNumHits();
					currentSeqEntry.addToeVals(eValue);
					currentSeqEntry.addToIdRatios(identityRATIO);
					currentSeqEntry.addToQueryCovRatios(queryCoverageRATIO);
					currentSeqEntry.addToGenomeLocs(genomeLoc);
					currentSeqEntry.addToGenomeSizes(genomeSize);
					currentSeqEntry.addToAccessionCodes(accessionCode);
					currentSeqEntry.addToGenuses(genusName);
					currentSeqEntry.addToSpecies(speciesName);
				}

			}
			fr1.close();

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tDone!");

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tPopulating species list and calculating genome coverage...");

			int numQueryTotal= 0;
			int numQueryPassingCO= 0;
			int numQueryPassingSpecies= 0;
			int numQueryFailingSpecies= 0;
			int numQueryPassingGenus= 0;
			int numQueryFailingGenus= 0;
			int numQueryPassingGroup= 0;
			int numQueryFailingGroup= 0;

			int readLength;
			double bestIdentRatio;
			double bestQueryCovRatio;
			String bestSpeciesHit;
			String bestGenusHit;
			String bestGroupHit;
			String bestKingdomHit;
			int bestGenomeSize;
			int genomeLoc;
			ArrayList<String> speciesNames= new ArrayList<String>();
			ArrayList<Species> speciesList= new ArrayList<Species>();	//for species-level classification
			ArrayList<String> genusNames= new ArrayList<String>();
			ArrayList<Genus> genusList= new ArrayList<Genus>();			//for genus-level classification
			ArrayList<String> groupNames= new ArrayList<String>();
			ArrayList<Group> groupList= new ArrayList<Group>(); 		//for group-level classification
			ArrayList<String> kingdomNames= new ArrayList<String>();
			ArrayList<Kingdom> kingdomList= new ArrayList<Kingdom>();
			for (SequenceEntry query : seqs) {
				boolean speciesLevelClassif= true; //can we do species level classification with this query?
				boolean genusLevelClassif= true;
				boolean groupLevelClassif= true;
				numQueryTotal++;
				readLength= query.getReadLength();
				bestIdentRatio= query.getIdRatios().get(0);
				bestQueryCovRatio= query.getQueryCovRatios().get(0);
				bestSpeciesHit= query.getSpecies().get(0);
				bestGenusHit= query.getGenuses().get(0);
				bestGroupHit= query.findGroup(query.getAccessionCodes().get(0), bestGenusHit, genomeTable);
				bestKingdomHit= query.findKingdom(query.getAccessionCodes().get(0), bestGenusHit, genomeTable);
				bestGenomeSize= query.getGenomeSizes().get(0);
				genomeLoc= query.getGenomeLocs().get(0);
				if ( (bestIdentRatio >= identityRatioCutOff) && (bestQueryCovRatio >= queryCoverageRatioCutOff) ) {
					numQueryPassingCO++;

					//consider species-level classification
					for (int i=1; i<query.getNumHits()-1; i++) { //POSSIBLE exception: check if ArrayList size > 1
						if ( !query.getSpecies().get(i).equals(bestSpeciesHit) && query.geteVals().get(i) <= query.getBestEvalue() ) {
							speciesLevelClassif= false;
							numQueryFailingSpecies++;
							//							break;
						}
					}

					if ( speciesLevelClassif == true ) {
						numQueryPassingSpecies++;
						String genusSpecies= bestGenusHit+bestSpeciesHit; //convoluted but done because the Species class is an older peice of code inherited for this new analysis
						if ( !speciesNames.contains(genusSpecies) ) {
							speciesNames.add(genusSpecies);
							Species s= new Species(bestGenusHit, bestSpeciesHit, bestGenomeSize, readLength);
							s.addGenomeLoc(genomeLoc);
							s.setNumReads(1);
							speciesList.add(s);
						} else {
							Species s= speciesList.get(speciesNames.indexOf(genusSpecies));
							s.addGenomeLoc(genomeLoc);
							s.setNumReads(s.getNumReads() + 1);
						}
					}

					//consider genus-level classification
					for (int i=1; i<query.getNumHits()-1; i++) {
						if (!query.getGenuses().get(i).equals(bestGenusHit) && query.geteVals().get(i) <= query.getBestEvalue() ) {
							genusLevelClassif= false;
							numQueryFailingGenus++;
						}
					}

					if ( genusLevelClassif == true ) {
						numQueryPassingGenus++;
						if ( !genusNames.contains(bestGenusHit) ) {
							genusNames.add(bestGenusHit);
							Genus g= new Genus(bestGenusHit, query);
							genusList.add(g);
						} else {
							Genus g= genusList.get(genusNames.indexOf(bestGenusHit));
							g.addToMatchingReads(query);
						}
					}




					//consider group-level classification
					for (int i=1; i<query.getNumHits()-1; i++) {
						String currentGroup= query.findGroup(query.getAccessionCodes().get(i), query.getGenuses().get(i), genomeTable);
						if ( !currentGroup.equals(bestGroupHit) && query.geteVals().get(i) <= query.getBestEvalue()) {
							groupLevelClassif= false;
							numQueryFailingGroup++;
						}
					}

					if ( groupLevelClassif == true ) {
						numQueryPassingGroup++;
						if ( !groupNames.contains(bestGroupHit) ) {
							groupNames.add(bestGroupHit);
							Group gr= new Group(bestGroupHit, query);
							groupList.add(gr);
						} else {
							Group gr= groupList.get(groupNames.indexOf(bestGroupHit));
							gr.addToMatchingReads(query);
						}
					}



					//consider kingdom-level classification
					
					//will only consider Kingdom classification of best hit.					
					if ( !kingdomNames.contains(bestKingdomHit)) {
						kingdomNames.add(bestKingdomHit);
						Kingdom k= new Kingdom(bestKingdomHit, query);
						kingdomList.add(k);
					} else {
						Kingdom k= kingdomList.get(kingdomNames.indexOf(bestKingdomHit));
						k.addToMatchingReads(query);
					}
				}

			}

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tDone!");

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tWriting to file...");

			out.write("\n\n\n*******************************\n\n\n");

			for (Species s : speciesList) {			
				out.write(s.getGenus() +"\t"+ s.getSpecies() +"\t"+ s.getGenomeSize() +"\t"+ s.getNumReads() +"\t"+ s.calcNumNucCoverage() +"\t"+ s.calcGenomeCoverage() + "\n");
			}

			out.write("\n\n\n*******************************\n\n\n");

			for (Genus g : genusList) {
				Species s= g.getSpeciesCalculations();
				out.write(g.getGenus() +"\t"+ g.getMatchingReads().size() +"\n");
				out.write(s.getGenus() +"\t"+ s.getSpecies() +"\t"+ s.getGenomeSize() +"\t"+ s.getNumReads() +"\t"+ s.calcNumNucCoverage() +"\t"+ s.calcGenomeCoverage() + "\n");				
			}

			out.write("\n\n\n*******************************\n\n\n");

			for (Group gr : groupList) {
				out.write(gr.getGroup() +"\t"+ gr.getMatchingReads().size() +"\n");
			}
			
			out.write("\n\n\n*******************************\n\n\n");
			
			for (Kingdom k : kingdomList) {
				out.write(k.getKingdom() + "\t"+ k.getMatchingReads().size() +"\n");
			}


			out.close();

			//Timer
			System.out.println("Time: "+((System.currentTimeMillis()-start)/1000F) + "\tDone!");

			System.out.println("Total number of query sequences: " + numQueryTotal);
			System.out.println("Number of query sequences passing the alignment cut-off: " + numQueryPassingCO);
			System.out.println("Number of query sequences used for species-level classification: " + numQueryPassingSpecies);
			System.out.println("Number of query sequences that could not be used in species-level classification: " + numQueryFailingSpecies);
			System.out.println("Number of query sequences used for genus-level classification: " + numQueryPassingGenus);
			System.out.println("Number of query sequences that could not be used in genus-level classification: " + numQueryFailingGenus);
			System.out.println("Number of query sequences used for group-level classification: " + numQueryPassingGroup);
			System.out.println("Number of query sequences that could not be used in group-level classification: " + numQueryFailingGroup);
			

		}catch (Exception e){//Catch exception if any
			System.err.println("Error: " + e.getMessage());
			e.printStackTrace();
		}
	}
}
