
import java.io.IOException;
import java.util.ArrayList;


public class SequenceEntry {

	private String seqName;
	private int numHits;
	private double bestEvalue;
	private int readLength;
	private ArrayList<String> kingdoms= new ArrayList<String>();
	private ArrayList<String> groups= new ArrayList<String>();
	private ArrayList<String> accessionCodes= new ArrayList<String>();
	private ArrayList<String> genuses= new ArrayList<String>();
	private ArrayList<String> species= new ArrayList<String>();
	private ArrayList<Double> eVals= new ArrayList<Double>();
	private ArrayList<Double> idRatios= new ArrayList<Double>();
	private ArrayList<Double> queryCovRatios= new ArrayList<Double>();
	private ArrayList<Integer> genomeLocs= new ArrayList<Integer>();
	private ArrayList<Integer> genomeSizes= new ArrayList<Integer>();
	
	public SequenceEntry(String sequenceName, double eVal, int lengthOfSequence) {
		setSeqName(sequenceName);
		setBestEvalue(eVal);
		numHits=1;
		setReadLength(lengthOfSequence);
	}
	
	
	
	public void incrNumHits() {
		this.numHits++;
	}
	
	public String findKingdom(String accession, String genus, String genomeTableLocation) throws IOException {
		String kingdom= "errorFindingKingdom";
		BasicParser listParser= new BasicParser(genomeTableLocation); // /xchip/pasteur/AlexK/databases/BacterialGenomes_info/complete_microbial_genomes_list_ncbi_condensed.txt
		String matchingLineA= listParser.stringMatcher(accession);
		String matchingLineG= listParser.stringMatcher(genus);
		
		if ( !(matchingLineA == null) ) {
			kingdom= BasicParser.splitLineAndReturnCol(matchingLineA, "\t", 1);
		} else if ( !(matchingLineG == null) ) {
			kingdom= BasicParser.splitLineAndReturnCol(matchingLineG, "\t", 1);
		}	
		
		return kingdom;
	}
	
	public String findGroup(String accession, String genus, String genomeTableLocation) throws IOException {
		String group= "errorFindingGroup";
		BasicParser listParser= new BasicParser(genomeTableLocation); // /xchip/pasteur/AlexK/databases/BacterialGenomes_info/complete_microbial_genomes_list_ncbi_condensed.txt
		String matchingLineA= listParser.stringMatcher(accession);
		String matchingLineG= listParser.stringMatcher(genus);
		
		if ( !(matchingLineA == null) ) {
			group= BasicParser.splitLineAndReturnCol(matchingLineA, "\t", 2);
		} else if ( !(matchingLineG == null) ) {
			group= BasicParser.splitLineAndReturnCol(matchingLineG, "\t", 2);
		}	
		
		return group;
	}

	
	
	
	
	
	
	
	
	//Getters and Setters (and Adders)
	
	public void setSeqName(String seqName) {
		this.seqName = seqName;
	}

	public String getSeqName() {
		return seqName;
	}

	public void setNumHits(int numHits) {
		this.numHits = numHits;
	}

	public int getNumHits() {
		return numHits;
	}

	public void setBestEvalue(double bestEvalue) {
		this.bestEvalue = bestEvalue;
	}

	public double getBestEvalue() {
		return bestEvalue;
	}

	public void addToGenuses(String newGenus) {
		this.genuses.add(newGenus);
	}

	public ArrayList<String> getGenuses() {
		return genuses;
	}

	public void addToSpecies(String newSpecies) {
		this.species.add(newSpecies);
	}

	public ArrayList<String> getSpecies() {
		return species;
	}

	public void addToeVals(double newEVal) {
		this.eVals.add(newEVal);
	}

	public ArrayList<Double> geteVals() {
		return eVals;
	}

	public void addToIdRatios(double newIdRatio) {
		this.idRatios.add(newIdRatio);
	}

	public ArrayList<Double> getIdRatios() {
		return idRatios;
	}

	public void addToQueryCovRatios(double newQueryCovRatio) {
		this.queryCovRatios.add(newQueryCovRatio);
	}

	public ArrayList<Double> getQueryCovRatios() {
		return queryCovRatios;
	}

	public void addToGenomeLocs(int genomeLoc) {
		this.genomeLocs.add(genomeLoc);
	}

	public ArrayList<Integer> getGenomeLocs() {
		return genomeLocs;
	}

	public void addToAccessionCodes(String accessionCode) {
		this.accessionCodes.add(accessionCode);
	}

	public ArrayList<String> getAccessionCodes() {
		return accessionCodes;
	}

	public void addToKingdoms(String kingdom) {
		this.kingdoms.add(kingdom);
	}

	public ArrayList<String> getKingdoms() {
		return kingdoms;
	}

	public void addToGroups(String group) {
		this.groups.add(group);
	}

	public ArrayList<String> getGroups() {
		return groups;
	}

	public void addToGenomeSizes(int genomeSize) {
		this.genomeSizes.add(genomeSize);
	}

	public ArrayList<Integer> getGenomeSizes() {
		return genomeSizes;
	}

	public void setReadLength(int readLength) {
		this.readLength = readLength;
	}

	public int getReadLength() {
		return readLength;
	}
}
